@@ -93,19 +93,15 @@ static bool refresh_provider_options()
   }
 }
 
-void wsrep_set_wsrep_on()
+void wsrep_set_wsrep_on(THD* thd)
 {
+  if (thd)
+    thd->wsrep_was_on= WSREP_ON_;
   WSREP_PROVIDER_EXISTS_= wsrep_provider &&
     strncasecmp(wsrep_provider, WSREP_NONE, FN_REFLEN);
   WSREP_ON_= global_system_variables.wsrep_on && WSREP_PROVIDER_EXISTS_;
 }
 
-/* This is intentionally declared as a weak global symbol, so that
-linking will succeed even if the server is built with a dynamically
-linked InnoDB. */
-ulong innodb_lock_schedule_algorithm __attribute__((weak));
-struct handlerton* innodb_hton_ptr __attribute__((weak));
-
 bool wsrep_on_update (sys_var *self, THD* thd, enum_var_type var_type)
 {
   if (var_type == OPT_GLOBAL)
@@ -134,7 +130,7 @@ bool wsrep_on_update (sys_var *self, THD* thd, enum_var_type var_type)
     thd->variables.wsrep_on= global_system_variables.wsrep_on= saved_wsrep_on;
   }
 
-  wsrep_set_wsrep_on();
+  wsrep_set_wsrep_on(thd);
 
   if (var_type == OPT_GLOBAL)
   {
@@ -158,14 +154,6 @@ bool wsrep_on_check(sys_var *self, THD* thd, set_var* var)
 
   if (new_wsrep_on)
   {
-    if (innodb_hton_ptr && innodb_lock_schedule_algorithm != 0)
-    {
-      my_message(ER_WRONG_ARGUMENTS, " WSREP (galera) can't be enabled "
-                 "if innodb_lock_schedule_algorithm=VATS. Please configure"
-                 " innodb_lock_schedule_algorithm=FCFS and restart.", MYF(0));
-      return true;
-    }
-
     if (!WSREP_PROVIDER_EXISTS)
     {
       my_message(ER_WRONG_ARGUMENTS, "WSREP (galera) can't be enabled "
@@ -520,7 +508,7 @@ bool wsrep_provider_update (sys_var *self, THD* thd, enum_var_type type)
   if (!rcode)
     refresh_provider_options();
 
-  wsrep_set_wsrep_on();
+  wsrep_set_wsrep_on(thd);
   mysql_mutex_lock(&LOCK_global_system_variables);
 
   return rcode;
@@ -540,7 +528,7 @@ void wsrep_provider_init (const char* value)
 
   if (wsrep_provider) my_free((void *)wsrep_provider);
   wsrep_provider= my_strdup(PSI_INSTRUMENT_MEM, value, MYF(0));
-  wsrep_set_wsrep_on();
+  wsrep_set_wsrep_on(NULL);
 }
 
 bool wsrep_provider_options_check(sys_var *self, THD* thd, set_var* var)
@@ -801,18 +789,25 @@ bool wsrep_slave_threads_update (sys_var *self, THD* thd, enum_var_type type)
   if (wsrep_slave_count_change > 0)
   {
     WSREP_DEBUG("Creating %d applier threads, total %ld", wsrep_slave_count_change, wsrep_slave_threads);
+    wsrep_thread_create_failed.store(false, std::memory_order_relaxed);
     res= wsrep_create_appliers(wsrep_slave_count_change, true);
     mysql_mutex_unlock(&LOCK_global_system_variables);
     mysql_mutex_unlock(&LOCK_wsrep_slave_threads);
     // Thread creation and execution is asyncronous, therefore we need
     // wait them to be started or error produced
-    while (wsrep_running_applier_threads != (ulong)wsrep_slave_threads)
+    while (wsrep_running_applier_threads != (ulong)wsrep_slave_threads &&
+	   !wsrep_thread_create_failed.load(std::memory_order_relaxed))
     {
 	    my_sleep(1000);
     }
 
     mysql_mutex_lock(&LOCK_global_system_variables);
 
+    if (wsrep_thread_create_failed.load(std::memory_order_relaxed)) {
+      wsrep_slave_threads= wsrep_running_applier_threads;
+      return true;
+    }
+
     WSREP_DEBUG("Running %lu applier threads", wsrep_running_applier_threads);
     wsrep_slave_count_change = 0;
   }
@@ -979,7 +974,7 @@ bool wsrep_max_ws_size_update(sys_var *self, THD *thd, enum_var_type)
 {
   char max_ws_size_opt[128];
   my_snprintf(max_ws_size_opt, sizeof(max_ws_size_opt),
-              "repl.max_ws_size=%d", wsrep_max_ws_size);
+              "repl.max_ws_size=%lu", wsrep_max_ws_size);
   enum wsrep::provider::status ret= Wsrep_server_state::instance().provider().options(max_ws_size_opt);
   if (ret)
   {
@@ -989,6 +984,11 @@ bool wsrep_max_ws_size_update(sys_var *self, THD *thd, enum_var_type)
   return refresh_provider_options();
 }
 
+bool wsrep_mode_check(sys_var *self, THD* thd, set_var* var)
+{
+  return false;
+}
+
 #if UNUSED /* eaec266eb16c (Sergei Golubchik  2014-09-28) */
 static SHOW_VAR wsrep_status_vars[]=
 {
@@ -1057,8 +1057,10 @@ static void export_wsrep_status_to_mysql(THD* thd)
 
 #if DYNAMIC
   if (wsrep_status_len != mysql_status_len) {
-    void* tmp= realloc (mysql_status_vars,
-                         (wsrep_status_len + 1) * sizeof(SHOW_VAR));
+    void* tmp= my_realloc(key_memory_WSREP,
+                          mysql_status_vars,
+                          (wsrep_status_len + 1) * sizeof(SHOW_VAR),
+                          MYF(MY_ALLOW_ZERO_PTR));
     if (!tmp) {
 
       sql_print_error ("Out of memory for wsrep status variables."
@@ -1110,6 +1112,15 @@ void wsrep_free_status (THD* thd)
   thd->wsrep_status_vars.clear();
 }
 
+void wsrep_free_status_vars()
+{
+#if DYNAMIC
+  my_free(mysql_status_vars);
+  mysql_status_vars= NULL;
+  mysql_status_len= 0;
+#endif
+}
+
 bool wsrep_gtid_domain_id_update(sys_var* self, THD *thd, enum_var_type)
 {
   WSREP_DEBUG("wsrep_gtid_domain_id_update: %llu",
@@ -1117,3 +1128,25 @@ bool wsrep_gtid_domain_id_update(sys_var* self, THD *thd, enum_var_type)
   wsrep_gtid_server.domain_id= wsrep_gtid_domain_id;
   return false;
 }
+
+bool wsrep_strict_ddl_update(sys_var *self, THD* thd, enum_var_type var_type)
+{
+  // In case user still sets wsrep_strict_ddl we set new
+  // option to wsrep_mode
+  if (wsrep_strict_ddl)
+    wsrep_mode|= WSREP_MODE_STRICT_REPLICATION;
+  else
+    wsrep_mode&= (~WSREP_MODE_STRICT_REPLICATION);
+  return false;
+}
+
+bool wsrep_replicate_myisam_update(sys_var *self, THD* thd, enum_var_type var_type)
+{
+  // In case user still sets wsrep_replicate_myisam we set new
+  // option to wsrep_mode
+  if (wsrep_replicate_myisam)
+    wsrep_mode|= WSREP_MODE_REPLICATE_MYISAM;
+  else
+    wsrep_mode&= (~WSREP_MODE_REPLICATE_MYISAM);
+  return false;
+}
