@@ -1,3 +1,18 @@
+/* Copyright (c) 2016, 2020, MariaDB Corporation.
+
+   This program is free software; you can redistribute it and/or modify
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation; version 2 of the License.
+
+   This program is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+   GNU General Public License for more details.
+
+   You should have received a copy of the GNU General Public License
+   along with this program; if not, write to the Free Software
+   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1335  USA */
+
 #include <my_global.h>
 #include <string.h>
 #include <m_ctype.h>
@@ -401,7 +416,7 @@ static int read_strn(json_engine_t *j)
     return 1;
 
   j->state= j->stack[j->stack_p];
-  j->value_len= (int)(j->s.c_str - j->value) - 1;
+  j->value_len= (int)(j->s.c_str - j->value) - j->s.c_next_len;
   return 0;
 }
 
@@ -467,12 +482,12 @@ enum json_num_states {
 
 static int json_num_states[NS_NUM_STATES][N_NUM_CLASSES]=
 {
-/*         -        +       0        1..9    POINT    E       END_OK   ERROR */
+/*         -        +       0         1..9    POINT    E       END_OK   ERROR */
 /*OK*/   { JE_SYN,  JE_SYN, JE_SYN,   JE_SYN, JE_SYN,  JE_SYN, JE_SYN, JE_BAD_CHR },
 /*GO*/   { NS_GO1,  JE_SYN, NS_Z,     NS_INT, JE_SYN,  JE_SYN, JE_SYN, JE_BAD_CHR },
 /*GO1*/  { JE_SYN,  JE_SYN, NS_Z1,    NS_INT, JE_SYN,  JE_SYN, JE_SYN, JE_BAD_CHR },
-/*ZERO*/ { JE_SYN,  JE_SYN, JE_SYN,   JE_SYN, NS_FRAC, JE_SYN, NS_OK,  JE_BAD_CHR },
-/*ZE1*/  { JE_SYN,  JE_SYN, JE_SYN,   JE_SYN, NS_FRAC, JE_SYN, NS_OK,  JE_BAD_CHR },
+/*ZERO*/ { JE_SYN,  JE_SYN, JE_SYN,   JE_SYN, NS_FRAC, NS_EX,  NS_OK,  JE_BAD_CHR },
+/*ZE1*/  { JE_SYN,  JE_SYN, JE_SYN,   JE_SYN, NS_FRAC, NS_EX,  NS_OK,  JE_BAD_CHR },
 /*INT*/  { JE_SYN,  JE_SYN, NS_INT,   NS_INT, NS_FRAC, NS_EX,  NS_OK,  JE_BAD_CHR },
 /*FRAC*/ { JE_SYN,  JE_SYN, NS_FRAC,  NS_FRAC,JE_SYN,  NS_EX,  NS_OK,  JE_BAD_CHR },
 /*EX*/   { NS_EX,   NS_EX,  NS_EX1,   NS_EX1, JE_SYN,  JE_SYN, JE_SYN, JE_BAD_CHR }, 
@@ -792,10 +807,13 @@ static json_state_handler json_actions[NR_JSON_STATES][NR_C_CLASSES]=
 int json_scan_start(json_engine_t *je,
                     CHARSET_INFO *i_cs, const uchar *str, const uchar *end)
 {
+  static const uchar no_time_to_die= 0;
+
   json_string_setup(&je->s, i_cs, str, end);
   je->stack[0]= JST_DONE;
   je->stack_p= 0;
   je->state= JST_VALUE;
+  je->killed_ptr = (uchar*)&no_time_to_die;
   return 0;
 }
 
@@ -956,7 +974,7 @@ int json_scan_next(json_engine_t *j)
   int t_next;
 
   get_first_nonspace(&j->s, &t_next, &j->sav_c_len);
-  return json_actions[j->state][t_next](j);
+  return *j->killed_ptr || json_actions[j->state][t_next](j);
 }
 
 
@@ -1511,8 +1529,7 @@ int json_append_ascii(CHARSET_INFO *json_cs,
   while (ascii < ascii_end)
   {
     int c_len;
-    if ((c_len= json_cs->cset->wc_mb(json_cs, (my_wc_t) *ascii,
-                                     json, json_end)) > 0)
+    if ((c_len= my_ci_wc_mb(json_cs, (my_wc_t) *ascii, json, json_end)) > 0)
     {
       json+= c_len;
       ascii++;
@@ -1538,7 +1555,7 @@ int json_unescape(CHARSET_INFO *json_cs,
   while (json_read_string_const_chr(&s) == 0)
   {
     int c_len;
-    if ((c_len= res_cs->cset->wc_mb(res_cs, s.c_next, res, res_end)) > 0)
+    if ((c_len= my_ci_wc_mb(res_cs, s.c_next, res, res_end)) > 0)
     {
       res+= c_len;
       continue;
@@ -1549,7 +1566,7 @@ int json_unescape(CHARSET_INFO *json_cs,
         Result charset doesn't support the json's character.
         Let's replace it with the '?' symbol.
       */
-      if ((c_len= res_cs->cset->wc_mb(res_cs, '?', res, res_end)) > 0)
+      if ((c_len= my_ci_wc_mb(res_cs, '?', res, res_end)) > 0)
       {
         res+= c_len;
         continue;
@@ -1608,14 +1625,14 @@ int json_escape(CHARSET_INFO *str_cs,
   {
     my_wc_t c_chr;
     int c_len;
-    if ((c_len= str_cs->cset->mb_wc(str_cs, &c_chr, str, str_end)) > 0)
+    if ((c_len= my_ci_mb_wc(str_cs, &c_chr, str, str_end)) > 0)
     {
       enum json_esc_char_classes c_class;
       
       str+= c_len;
       if (c_chr >= 0x60 || (c_class= json_escape_chr_map[c_chr]) == ESC_)
       {
-        if ((c_len= json_cs->cset->wc_mb(json_cs, c_chr, json, json_end)) > 0)
+        if ((c_len= my_ci_wc_mb(json_cs, c_chr, json, json_end)) > 0)
         {
           json+= c_len;
           continue;
@@ -1630,9 +1647,8 @@ int json_escape(CHARSET_INFO *str_cs,
         c_class= ESC_U;
       }
 
-      if ((c_len= json_cs->cset->wc_mb(json_cs, '\\', json, json_end)) <= 0 ||
-          (c_len= json_cs->cset->wc_mb(json_cs,
-                                       (c_class == ESC_BS) ? c_chr : c_class,
+      if ((c_len= my_ci_wc_mb(json_cs, '\\', json, json_end)) <= 0 ||
+          (c_len= my_ci_wc_mb(json_cs, (c_class == ESC_BS) ? c_chr : c_class,
                                        json+= c_len, json_end)) <= 0)
       {
         /* JSON buffer is depleted. */
