@@ -273,8 +273,8 @@ int wsrep::transaction::after_row()
     return ret;
 }
 
-int wsrep::transaction::before_prepare(
-    wsrep::unique_lock<wsrep::mutex>& lock)
+int wsrep::transaction::before_prepare(wsrep::unique_lock<wsrep::mutex>& lock,
+                                       const wsrep::provider::seq_cb_t* seq_cb)
 {
     assert(lock.owns_lock());
     int ret(0);
@@ -349,7 +349,7 @@ int wsrep::transaction::before_prepare(
             }
             else
             {
-                ret = certify_commit(lock);
+                ret = certify_commit(lock, seq_cb);
             }
 
             assert((ret == 0 && state() == s_preparing) ||
@@ -445,7 +445,7 @@ int wsrep::transaction::after_prepare(
     return ret;
 }
 
-int wsrep::transaction::before_commit()
+int wsrep::transaction::before_commit(const wsrep::provider::seq_cb* seq_cb)
 {
     int ret(1);
 
@@ -465,7 +465,7 @@ int wsrep::transaction::before_commit()
     case wsrep::client_state::m_local:
         if (state() == s_executing)
         {
-            ret = before_prepare(lock) || after_prepare(lock);
+            ret = before_prepare(lock, seq_cb) || after_prepare(lock);
             assert((ret == 0 &&
                     (state() == s_committing || state() == s_prepared))
                    ||
@@ -495,7 +495,7 @@ int wsrep::transaction::before_commit()
 
         if (ret == 0 && state() == s_prepared)
         {
-            ret = certify_commit(lock);
+            ret = certify_commit(lock, nullptr);
             assert((ret == 0 && state() == s_committing) ||
                    (state() == s_must_abort ||
                     state() == s_must_replay ||
@@ -543,7 +543,7 @@ int wsrep::transaction::before_commit()
         }
         else if (state() == s_executing || state() == s_replaying)
         {
-            ret = before_prepare(lock) || after_prepare(lock);
+            ret = before_prepare(lock, nullptr) || after_prepare(lock);
         }
         else
         {
@@ -1195,7 +1195,7 @@ int wsrep::transaction::commit_or_rollback_by_xid(const wsrep::xid& xid,
         provider().certify(client_state_.id(),
                            ws_handle_,
                            flags(),
-                           meta));
+                           meta, nullptr));
 
     int ret;
     if (cert_ret == wsrep::provider::success)
@@ -1622,7 +1622,7 @@ int wsrep::transaction::certify_fragment(
             cert_ret = provider().certify(client_state_.id(),
                                           ws_handle_,
                                           flags(),
-                                          sr_ws_meta);
+                                          sr_ws_meta, nullptr);
             client_service_.debug_crash(
                 "crash_replicate_fragment_after_certify");
 
@@ -1744,7 +1744,7 @@ int wsrep::transaction::certify_fragment(
 }
 
 int wsrep::transaction::certify_commit(
-    wsrep::unique_lock<wsrep::mutex>& lock)
+    wsrep::unique_lock<wsrep::mutex>& lock, const provider::seq_cb_t* seq_cb)
 {
     assert(lock.owns_lock());
     assert(active());
@@ -1828,7 +1828,7 @@ int wsrep::transaction::certify_commit(
         cert_ret(provider().certify(client_state_.id(),
                                    ws_handle_,
                                    flags(),
-                                   ws_meta_));
+                                   ws_meta_, seq_cb));
     client_service_.debug_sync("wsrep_after_certification");
 
     lock.lock();
